@@ -1,11 +1,13 @@
+use jsonpath_rust::JsonPathQuery;
 use regex::Regex;
+use serde_json::Value;
+use std::cmp::{Eq, Ord, Ordering, PartialEq, PartialOrd};
+use std::fs;
+use std::path::Path;
 use std::str::FromStr;
 use std::string::ToString;
 use std::{error::Error, fmt};
 
-use jsonpath_rust::JsonPathQuery;
-use serde_json::{json, Value};
-
 #[cfg(feature = "jq")]
 use jq_rs;
 
@@ -151,18 +153,27 @@ impl BarbFilter {
         &self.name
     }
 
+    #[cfg(test)]
     pub fn filter(&self) -> &String {
         &self.filter
     }
 
     #[cfg(feature = "jq")]
     fn apply_jq(&self, body: &String) -> Result<String, String> {
+        if let FilterType::JQ = self.filter_type {
+            return Err(String::from("Incorrect filter type"));
+        }
+
         jq_rs::run(self.filter.as_str(), body.as_str())
             .map_err(|x| x.to_string())
             .map(|x| String::from(x.trim().trim_matches('"')))
     }
 
     fn apply_path(&self, body: &String) -> Result<String, String> {
+        if let FilterType::JQ = self.filter_type {
+            return Err(String::from("Incorrect filter type"));
+        }
+
         let json: Value = serde_json::from_str(body.as_str())
             .map_err(|_| String::from("Failed to decode body"))?;
         let path = &json.path(self.filter.as_str())?;
@@ -194,22 +205,29 @@ struct BarbPreamble {
     pub url: String,
     pub headers: Vec<Header>,
     pub filters: Vec<BarbFilter>,
-    //pub filter: Option<BarbFilter>,
+    pub dependency: Option<String>,
 }
 
 impl BarbPreamble {
-    fn new(method: Method, url: String, headers: Vec<Header>, filters: Vec<BarbFilter>) -> Self {
+    fn new(
+        method: Method,
+        url: String,
+        headers: Vec<Header>,
+        filters: Vec<BarbFilter>,
+        dependency: Option<String>,
+    ) -> Self {
         BarbPreamble {
             method,
             url,
             headers,
-            //filter: Vec<BarbFilter>,
             filters,
+            dependency,
         }
     }
 }
 
 pub struct BarbFile {
+    file_name: String,
     preamble: BarbPreamble,
     body: Option<String>,
 }
@@ -238,6 +256,40 @@ impl BarbFile {
     pub fn body(&self) -> &Option<String> {
         &self.body
     }
+
+    pub fn dependency(&self) -> Option<String> {
+        let dep = self.preamble.dependency.as_ref()?;
+        let dep_path = Path::new(dep);
+
+        if !dep_path.is_absolute() {
+            let my_path = Path::new(&self.file_name)
+                .parent()
+                .or(Some(Path::new("")))?;
+            return Some(String::from(my_path.join(dep_path).to_str()?));
+        }
+
+        Some(String::from(dep_path.to_str()?))
+    }
+}
+
+impl PartialEq for BarbFile {
+    fn eq(&self, other: &Self) -> bool {
+        self.file_name == other.file_name
+    }
+}
+
+impl Eq for BarbFile {}
+
+impl PartialOrd for BarbFile {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for BarbFile {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.file_name.cmp(&other.file_name)
+    }
 }
 
 fn decode_url_line(line: &str) -> Result<(Method, String), BarbParseError> {
@@ -257,6 +309,19 @@ fn decode_header(line: &str) -> Result<Header, BarbParseError> {
     })
 }
 
+impl BarbFile {
+    pub fn from_file(file_name: String) -> Result<Self, BarbParseError> {
+        let mut bfile = Self::from_str(
+            fs::read_to_string(file_name.as_str())
+                .map_err(|_| BarbParseError {})?
+                .as_str(),
+        )
+        .map_err(|_| BarbParseError {})?;
+        bfile.file_name = file_name;
+        Ok(bfile)
+    }
+}
+
 impl FromStr for BarbFile {
     type Err = BarbParseError;
 
@@ -265,6 +330,7 @@ impl FromStr for BarbFile {
         let (method, url) = decode_url_line(lines.next().ok_or(BarbParseError {})?)?;
         let mut headers: Vec<Header> = vec![];
         let mut filters: Vec<BarbFilter> = vec![];
+        let mut dependency: Option<String> = None;
 
         for line in &mut lines {
             if line == "" {
@@ -276,6 +342,10 @@ impl FromStr for BarbFile {
                 headers.push(decode_header(line).map_err(|_| BarbParseError {})?);
             }
 
+            if let Some('>') = line.chars().nth(1) {
+                dependency = line.get(2..).map(|x| String::from(x));
+            }
+
             if BarbFilter::is_match(String::from(line)) {
                 match BarbFilter::from_str(line) {
                     Ok(filter) => filters.push(filter),
@@ -287,7 +357,8 @@ impl FromStr for BarbFile {
         let body = lines.fold(String::from(""), |acc, x| acc + x);
 
         Ok(BarbFile {
-            preamble: BarbPreamble::new(method, url, headers, filters),
+            file_name: String::from(""),
+            preamble: BarbPreamble::new(method, url, headers, filters, dependency),
             body: if body == "" { None } else { Some(body) },
         })
     }
@@ -299,7 +370,6 @@ mod tests {
 
     #[test]
     fn test_method_from_str() {
-        assert!(matches!(Method::from_str("GET").unwrap(), Method::GET));
         assert!(matches!(Method::from_str("GET").unwrap(), Method::GET));
         assert!(matches!(Method::from_str("PUT").unwrap(), Method::PUT));
         assert!(matches!(Method::from_str("POST").unwrap(), Method::POST));
@@ -310,6 +380,15 @@ mod tests {
         ));
     }
 
+    #[test]
+    fn test_method_takes_body() {
+        assert!(!Method::GET.takes_body());
+        assert!(Method::PUT.takes_body());
+        assert!(Method::POST.takes_body());
+        assert!(Method::PATCH.takes_body());
+        assert!(!Method::DELETE.takes_body());
+    }
+
     #[test]
     fn test_decode_url_line() {
         let (method, url) = decode_url_line("#GET^http://blahblah").unwrap();
@@ -359,16 +438,68 @@ mod tests {
     }
 
     #[test]
-    fn test_parse_named_filter() {
+    fn test_jq_parse_named_filter() {
         let filter = BarbFilter::from_str("#FOO|.bar.foo").unwrap();
         assert_eq!(filter.name, Some(String::from("FOO")));
         assert_eq!(filter.filter, String::from(".bar.foo"));
+        assert!(matches!(filter.filter_type, FilterType::JQ));
     }
 
     #[test]
-    fn test_parse_named_filter_no_name() {
+    fn test_jq_parse_named_filter_no_name() {
         let filter = BarbFilter::from_str("#|.bar.foo").unwrap();
         assert_eq!(filter.name, None);
         assert_eq!(filter.filter, String::from(".bar.foo"));
+        assert!(matches!(filter.filter_type, FilterType::JQ));
+    }
+
+    #[test]
+    fn test_path_parse_named_filter() {
+        let filter = BarbFilter::from_str("#FOO$$.bar.foo").unwrap();
+        assert_eq!(filter.name, Some(String::from("FOO")));
+        assert_eq!(filter.filter, String::from("$.bar.foo"));
+        assert!(matches!(filter.filter_type, FilterType::PATH));
+    }
+
+    #[test]
+    fn test_path_parse_named_filter_no_name() {
+        let filter = BarbFilter::from_str("#$$.bar.foo").unwrap();
+        assert_eq!(filter.name, None);
+        assert_eq!(filter.filter, String::from("$.bar.foo"));
+        assert!(matches!(filter.filter_type, FilterType::PATH));
+    }
+
+    #[cfg(feature = "jq")]
+    #[test]
+    fn test_apply_filter_jq() {
+        let jq_f = BarbFilter::from_str("#|.status").unwrap();
+        let subject = String::from(r#"{"status": "OK"}"#);
+        assert_eq!(jq_f.apply(&subject).unwrap(), String::from("OK"));
+    }
+
+    #[test]
+    fn test_apply_filter_path() {
+        let path_f = BarbFilter::from_str("#$$.status").unwrap();
+        let subject = String::from(r#"{"status": "OK"}"#);
+        assert_eq!(path_f.apply(&subject).unwrap(), String::from("OK"));
+    }
+
+    #[test]
+    fn test_parse_dependency() {
+        let bfile = BarbFile::from_str("#GET^http://test.com\n#>blah.barb").unwrap();
+        assert_eq!(
+            bfile.dependency().as_ref().unwrap(),
+            &String::from("blah.barb")
+        );
+    }
+
+    #[test]
+    fn test_parse_mult_dependency_keeps_last() {
+        let bfile = BarbFile::from_str("#GET^http://test.com\n#>blah.barb\n#>foo.barb\n#>bar.barb")
+            .unwrap();
+        assert_eq!(
+            bfile.dependency().as_ref().unwrap(),
+            &String::from("bar.barb")
+        );
     }
 }
